@@ -25,7 +25,7 @@ from pycaret.internal.utils import (
 from pycaret.internal.distributions import *
 import pycaret.containers.base_container
 import numpy as np
-
+from packaging import version
 
 class ClassifierContainer(ModelContainer):
     """
@@ -749,14 +749,22 @@ class RandomForestClassifierContainer(ClassifierContainer):
                 pycaret.internal.cuml_wrappers.get_random_forest_classifier()
             )
 
-        args = (
-            {
+        if not gpu_imported:
+            args = {
                 "random_state": globals_dict["seed"],
                 "n_jobs": globals_dict["n_jobs_param"],
             }
-            if not gpu_imported
-            else {"seed": globals_dict["seed"]}
-        )
+        else:
+            import cuml
+            if version.parse(cuml.__version__) >= version.parse("0.19"):
+                args = {
+                "random_state": globals_dict["seed"],
+                }
+            else:
+                args = {
+                    "seed": globals_dict["seed"]
+                }
+        
         tune_args = {}
         tune_grid = {
             "n_estimators": np_list_arange(10, 300, 10, inclusive=True),
@@ -1082,10 +1090,9 @@ class XGBClassifierContainer(ClassifierContainer):
             self.active = False
             return
 
-        xgboost_version = tuple([int(x) for x in xgboost.__version__.split(".")])
-        if xgboost_version < (1, 1, 0):
+        if version.parse(xgboost.__version__) < version.parse("1.1.0"):
             logger.warning(
-                f"Wrong xgboost version. Expected xgboost>=1.1.0, got xgboost=={xgboost_version}"
+                f"Wrong xgboost version. Expected xgboost>=1.1.0, got xgboost=={xgboost.__version__}"
             )
             self.active = False
             return
@@ -1318,18 +1325,26 @@ class LGBMClassifierContainer(ClassifierContainer):
             try:
                 lgb = LGBMClassifier(device="gpu")
                 lgb.fit(np.zeros((2, 2)), [0, 1])
-                is_gpu_enabled = True
+                is_gpu_enabled = "gpu"
                 del lgb
-            except LightGBMError:
-                is_gpu_enabled = False
-                if globals_dict["gpu_param"] == "force":
-                    raise RuntimeError(
-                        f"LightGBM GPU mode not available. Consult https://lightgbm.readthedocs.io/en/latest/GPU-Tutorial.html."
-                    )
+            except:
+                try:
+                    lgb = LGBMClassifier(device="cuda")
+                    lgb.fit(np.zeros((2, 2)), [0, 1])
+                    is_gpu_enabled = "cuda"
+                    del lgb
+                except LightGBMError:
+                    is_gpu_enabled = False
+                    if globals_dict["gpu_param"] == "force":
+                        raise RuntimeError(
+                            f"LightGBM GPU mode not available. Consult https://lightgbm.readthedocs.io/en/latest/GPU-Tutorial.html."
+                        )
 
-        if is_gpu_enabled:
+        if is_gpu_enabled=="gpu":
             args["device"] = "gpu"
-
+        elif is_gpu_enabled=="cuda":
+            args["device"] = "cuda"
+        
         super().__init__(
             id="lightgbm",
             name="Light Gradient Boosting Machine",
@@ -1354,10 +1369,9 @@ class CatBoostClassifierContainer(ClassifierContainer):
             self.active = False
             return
 
-        catboost_version = tuple([int(x) for x in catboost.__version__.split(".")])
-        if catboost_version < (0, 23, 2):
+        if version.parse(catboost.__version__) < version.parse("0.23.2"):
             logger.warning(
-                f"Wrong catboost version. Expected catboost>=0.23.2, got catboost=={catboost_version}"
+                f"Wrong catboost version. Expected catboost>=0.23.2, got catboost=={catboost.__version__}"
             )
             self.active = False
             return
