@@ -257,7 +257,7 @@ def setup(
     pca_method: str, default = 'linear'
         The 'linear' method performs uses Singular Value  Decomposition. Other options are:
         
-        - kernel: dimensionality reduction through the use of RVF kernel.
+        - kernel: dimensionality reduction through the use of RBF kernel.
         - incremental: replacement for 'linear' pca when the dataset is too large.
 
 
@@ -306,7 +306,8 @@ def setup(
     remove_multicollinearity: bool, default = False
         When set to True, features with the inter-correlations higher than the defined 
         threshold are removed. When two features are highly correlated with each other, 
-        the feature that is less correlated with the target variable is removed. 
+        the feature that is less correlated with the target variable is removed. Only
+        considers numeric features.
 
 
     multicollinearity_threshold: float, default = 0.9
@@ -1405,7 +1406,7 @@ def plot_model(
     groups: Optional[Union[str, Any]] = None,
     use_train_data: bool = False,
     verbose: bool = True,
-    display_format: Optional[str] = None
+    display_format: Optional[str] = None,
 ) -> str:
 
     """
@@ -1500,7 +1501,7 @@ def plot_model(
         verbose=verbose,
         use_train_data=use_train_data,
         system=True,
-        display_format=display_format
+        display_format=display_format,
     )
 
 
@@ -1578,14 +1579,16 @@ def interpret_model(
     feature: Optional[str] = None,
     observation: Optional[int] = None,
     use_train_data: bool = False,
+    X_new_sample: Optional[pd.DataFrame] = None,
+    y_new_sample: Optional[pd.DataFrame] = None,  # add for pfi explainer
     save: bool = False,
     **kwargs,
 ):
 
     """
-    This function analyzes the predictions generated from a tree-based model. It is
-    implemented based on the SHAP (SHapley Additive exPlanations). For more info on
-    this, please see https://shap.readthedocs.io/en/latest/
+    This function analyzes the predictions generated from a trained model. Most plots
+    in this function are implemented based on the SHAP (SHapley Additive exPlanations).
+    For more info on this, please see https://shap.readthedocs.io/en/latest/
 
 
     Example
@@ -1602,19 +1605,31 @@ def interpret_model(
         Trained model object
 
 
-    plot: str, default = 'summary'
-        Type of plot. Available options are: 'summary', 'correlation', and 'reason'.
+    plot : str, default = 'summary'
+        Abbreviation of type of plot. The current list of plots supported 
+        are (Plot - Name):
+
+        * 'summary' - Summary Plot using SHAP
+        * 'correlation' - Dependence Plot using SHAP
+        * 'reason' - Force Plot using SHAP
+        * 'pdp' - Partial Dependence Plot
+        * 'msa' - Morris Sensitivity Analysis
+        * 'pfi' - Permutation Feature Importance
 
 
     feature: str, default = None
-        Feature to check correlation with. This parameter is only required when ``plot``
-        type is 'correlation'. When set to None, it uses the first column in the train
-        dataset.
+        This parameter is only needed when plot = 'correlation' or 'pdp'. 
+        By default feature is set to None which means the first column of the 
+        dataset will be used as a variable. A feature parameter must be passed 
+        to change this.
 
 
-    observation: int, default = None
-        Observation index number in holdout set to explain. When ``plot`` is not
-        'reason', this parameter is ignored. 
+    observation: integer, default = None
+        This parameter only comes into effect when plot is set to 'reason'. If no
+        observation number is provided, it will return an analysis of all observations
+        with the option to select the feature on x and y axes through drop down
+        interactivity. For analysis at the sample level, an observation parameter must
+        be passed with the index value of the observation in test / hold-out set.
 
 
     use_train_data: bool, default = False
@@ -1622,6 +1637,18 @@ def interpret_model(
         of test data.
 
 
+    X_new_sample: pd.DataFrame, default = None
+        Row from an out-of-sample dataframe (neither train nor test data) to be plotted.
+        The sample must have the same columns as the raw input data, and it is transformed
+        by the preprocessing pipeline automatically before plotting.
+
+
+    y_new_sample: pd.DataFrame, default = None
+        Row from an out-of-sample dataframe (neither train nor test data) to be plotted.
+        The sample must have the same columns as the raw input label data, and it is transformed
+        by the preprocessing pipeline automatically before plotting.
+
+
     save: bool, default = False
         When set to True, Plot is saved as a 'png' file in current working directory.
 
@@ -1641,6 +1668,8 @@ def interpret_model(
         feature=feature,
         observation=observation,
         use_train_data=use_train_data,
+        X_new_sample=X_new_sample,
+        y_new_sample=y_new_sample,
         save=save,
         **kwargs,
     )
@@ -1770,11 +1799,10 @@ def finalize_model(
 def deploy_model(
     model, model_name: str, authentication: dict, platform: str = "aws",
 ):
-
     """
     This function deploys the transformation pipeline and trained model on cloud.
-    
-    
+
+
     Example
     -------
     >>> from pycaret.datasets import get_data
@@ -1782,46 +1810,50 @@ def deploy_model(
     >>> from pycaret.regression import *
     >>> exp_name = setup(data = boston,  target = 'medv')
     >>> lr = create_model('lr')
+    >>> # sets appropriate credentials for the platform as environment variables
+    >>> import os
+    >>> os.environ["AWS_ACCESS_KEY_ID"] = str("foo")
+    >>> os.environ["AWS_SECRET_ACCESS_KEY"] = str("bar")
     >>> deploy_model(model = lr, model_name = 'lr-for-deployment', platform = 'aws', authentication = {'bucket' : 'S3-bucket-name'})
-        
+
 
     Amazon Web Service (AWS) users:
-        To deploy a model on AWS S3 ('aws'), environment variables must be set in your
-        local environment. To configure AWS environment variables, type ``aws configure`` 
-        in the command line. Following information from the IAM portal of amazon console 
-        account is required:
+        To deploy a model on AWS S3 ('aws'), the credentials have to be passed. The easiest way is to use environment
+        variables in your local environment. Following information from the IAM portal of amazon console account
+        are required:
 
         - AWS Access Key ID
         - AWS Secret Key Access
-        - Default Region Name (can be seen under Global settings on your AWS console)
 
-        More info: https://docs.aws.amazon.com/cli/latest/userguide/cli-configure-envvars.html
+        More info: https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html#environment-variables
 
 
     Google Cloud Platform (GCP) users:
-        To deploy a model on Google Cloud Platform ('gcp'), project must be created 
-        using command line or GCP console. Once project is created, you must create 
-        a service account and download the service account key as a JSON file to set 
-        environment variables in your local environment. 
+        To deploy a model on Google Cloud Platform ('gcp'), project must be created
+        using command line or GCP console. Once project is created, you must create
+        a service account and download the service account key as a JSON file to set
+        environment variables in your local environment.
 
         More info: https://cloud.google.com/docs/authentication/production
 
-    
+
     Microsoft Azure (Azure) users:
         To deploy a model on Microsoft Azure ('azure'), environment variables for connection
         string must be set in your local environment. Go to settings of storage account on
-        Azure portal to access the connection string required. 
+        Azure portal to access the connection string required.
+
+        - AZURE_STORAGE_CONNECTION_STRING (required as environment variable)
 
         More info: https://docs.microsoft.com/en-us/azure/storage/blobs/storage-quickstart-blobs-python?toc=%2Fpython%2Fazure%2FTOC.json
 
 
     model: scikit-learn compatible object
         Trained model object
-    
+
 
     model_name: str
         Name of model.
-    
+
 
     authentication: dict
         Dictionary of applicable authentication tokens.
@@ -1834,15 +1866,15 @@ def deploy_model(
 
         When platform = 'azure':
         {'container': 'azure-container-name'}
-    
+
 
     platform: str, default = 'aws'
         Name of the platform. Currently supported platforms: 'aws', 'gcp' and 'azure'.
-    
+
 
     Returns:
         None
-    
+
     """
 
     return pycaret.internal.tabular.deploy_model(
@@ -1853,7 +1885,9 @@ def deploy_model(
     )
 
 
-def save_model(model, model_name: str, model_only: bool = False, verbose: bool = True):
+def save_model(
+    model, model_name: str, model_only: bool = False, verbose: bool = True, **kwargs
+):
 
     """
     This function saves the transformation pipeline and trained model object 
@@ -1882,6 +1916,10 @@ def save_model(model, model_name: str, model_only: bool = False, verbose: bool =
         entire pipeline.
 
 
+    **kwargs: 
+        Additional keyword arguments to pass to joblib.dump().
+
+
     verbose: bool, default = True
         Success message is not printed when verbose is set to False.
 
@@ -1892,7 +1930,11 @@ def save_model(model, model_name: str, model_only: bool = False, verbose: bool =
     """
 
     return pycaret.internal.tabular.save_model(
-        model=model, model_name=model_name, model_only=model_only, verbose=verbose
+        model=model,
+        model_name=model_name,
+        model_only=model_only,
+        verbose=verbose,
+        **kwargs,
     )
 
 
